@@ -272,13 +272,13 @@ func TestTemplateExpansion(t *testing.T) {
 		{
 			// Humanize - int.
 			text:   "{{ range . }}{{ humanize . }}:{{ end }}",
-			input:  []int{0, -1, 1, 1234567, math.MaxInt64},
+			input:  []int64{0, -1, 1, 1234567, math.MaxInt64},
 			output: "0:-1:1:1.235M:9.223E:",
 		},
 		{
 			// Humanize - uint.
 			text:   "{{ range . }}{{ humanize . }}:{{ end }}",
-			input:  []uint{0, 1, 1234567, math.MaxUint64},
+			input:  []uint64{0, 1, 1234567, math.MaxUint64},
 			output: "0:1:1.235M:18.45E:",
 		},
 		{
@@ -302,13 +302,13 @@ func TestTemplateExpansion(t *testing.T) {
 		{
 			// Humanize1024 - int.
 			text:   "{{ range . }}{{ humanize1024 . }}:{{ end }}",
-			input:  []int{0, -1, 1, 1234567, math.MaxInt64},
+			input:  []int64{0, -1, 1, 1234567, math.MaxInt64},
 			output: "0:-1:1:1.177Mi:8Ei:",
 		},
 		{
 			// Humanize1024 - uint.
 			text:   "{{ range . }}{{ humanize1024 . }}:{{ end }}",
-			input:  []uint{0, 1, 1234567, math.MaxUint64},
+			input:  []uint64{0, 1, 1234567, math.MaxUint64},
 			output: "0:1:1.177Mi:16Ei:",
 		},
 		{
@@ -373,13 +373,13 @@ func TestTemplateExpansion(t *testing.T) {
 		{
 			// HumanizePercentage - int.
 			text:   "{{ range . }}{{ humanizePercentage . }}:{{ end }}",
-			input:  []int{0, -1, 1, 1234567, math.MaxInt64},
+			input:  []int64{0, -1, 1, 1234567, math.MaxInt64},
 			output: "0%:-100%:100%:1.235e+08%:9.223e+20%:",
 		},
 		{
 			// HumanizePercentage - uint.
 			text:   "{{ range . }}{{ humanizePercentage . }}:{{ end }}",
-			input:  []uint{0, 1, 1234567, math.MaxUint64},
+			input:  []uint64{0, 1, 1234567, math.MaxUint64},
 			output: "0%:100%:1.235e+08%:1.845e+21%:",
 		},
 		{
@@ -396,26 +396,26 @@ func TestTemplateExpansion(t *testing.T) {
 		{
 			// HumanizeTimestamp - int.
 			text:   "{{ range . }}{{ humanizeTimestamp . }}:{{ end }}",
-			input:  []int{0, -1, 1, 1234567, 9223372036},
+			input:  []int64{0, -1, 1, 1234567, 9223372036},
 			output: "1970-01-01 00:00:00 +0000 UTC:1969-12-31 23:59:59 +0000 UTC:1970-01-01 00:00:01 +0000 UTC:1970-01-15 06:56:07 +0000 UTC:2262-04-11 23:47:16 +0000 UTC:",
 		},
 		{
 			// HumanizeTimestamp - uint.
 			text:   "{{ range . }}{{ humanizeTimestamp . }}:{{ end }}",
-			input:  []uint{0, 1, 1234567, 9223372036},
+			input:  []uint64{0, 1, 1234567, 9223372036},
 			output: "1970-01-01 00:00:00 +0000 UTC:1970-01-01 00:00:01 +0000 UTC:1970-01-15 06:56:07 +0000 UTC:2262-04-11 23:47:16 +0000 UTC:",
 		},
 		{
 			// HumanizeTimestamp - int with error.
 			text:       "{{ range . }}{{ humanizeTimestamp . }}:{{ end }}",
-			input:      []int{math.MinInt64, math.MaxInt64},
+			input:      []int64{math.MinInt64, math.MaxInt64},
 			shouldFail: true,
 			errorMsg:   `error executing template test: template: test:1:16: executing "test" at <humanizeTimestamp .>: error calling humanizeTimestamp: -9.223372036854776e+18 cannot be represented as a nanoseconds timestamp since it overflows int64`,
 		},
 		{
 			// HumanizeTimestamp - uint with error.
 			text:       "{{ range . }}{{ humanizeTimestamp . }}:{{ end }}",
-			input:      []uint{math.MaxUint64},
+			input:      []uint64{math.MaxUint64},
 			shouldFail: true,
 			errorMsg:   `error executing template test: template: test:1:16: executing "test" at <humanizeTimestamp .>: error calling humanizeTimestamp: 1.8446744073709552e+19 cannot be represented as a nanoseconds timestamp since it overflows int64`,
 		},
@@ -480,6 +480,41 @@ func TestTemplateExpansion(t *testing.T) {
 			text:   "{{ printf \"%0.2f\" (parseDuration \"1h2m10ms\") }}",
 			output: "3720.01",
 		},
+		{
+			// Simple hostname.
+			text:   "{{ \"foo.example.com\" | stripDomain }}",
+			output: "foo",
+		},
+		{
+			// Hostname with port.
+			text:   "{{ \"foo.example.com:12345\" | stripDomain }}",
+			output: "foo:12345",
+		},
+		{
+			// Simple IPv4 address.
+			text:   "{{ \"192.0.2.1\" | stripDomain }}",
+			output: "192.0.2.1",
+		},
+		{
+			// IPv4 address with port.
+			text:   "{{ \"192.0.2.1:12345\" | stripDomain }}",
+			output: "192.0.2.1:12345",
+		},
+		{
+			// Simple IPv6 address.
+			text:   "{{ \"2001:0DB8::1\" | stripDomain }}",
+			output: "2001:0DB8::1",
+		},
+		{
+			// IPv6 address with port.
+			text:   "{{ \"[2001:0DB8::1]:12345\" | stripDomain }}",
+			output: "[2001:0DB8::1]:12345",
+		},
+		{
+			// Value can't be split into host and port.
+			text:   "{{ \"[2001:0DB8::1]::12345\" | stripDomain }}",
+			output: "[2001:0DB8::1]::12345",
+		},
 	})
 }
 
