@@ -15,6 +15,7 @@ package agent
 
 import (
 	"context"
+	"fmt"
 	"path/filepath"
 	"strconv"
 	"testing"
@@ -24,6 +25,7 @@ import (
 
 	"github.com/prometheus/client_golang/prometheus"
 	dto "github.com/prometheus/client_model/go"
+	"github.com/prometheus/common/model"
 	"github.com/stretchr/testify/require"
 
 	"github.com/prometheus/prometheus/model/exemplar"
@@ -127,7 +129,7 @@ func TestCommit(t *testing.T) {
 
 			e := exemplar.Exemplar{
 				Labels: lset,
-				Ts:     sample[0].T(),
+				Ts:     sample[0].T() + int64(i),
 				Value:  sample[0].V(),
 				HasTs:  true,
 			}
@@ -409,6 +411,41 @@ func TestLockfile(t *testing.T) {
 	})
 }
 
+func Test_ExistingWAL_NextRef(t *testing.T) {
+	dbDir := t.TempDir()
+	rs := remote.NewStorage(log.NewNopLogger(), nil, startTime, dbDir, time.Second*30, nil)
+	defer func() {
+		require.NoError(t, rs.Close())
+	}()
+
+	db, err := Open(log.NewNopLogger(), nil, rs, dbDir, DefaultOptions())
+	require.NoError(t, err)
+
+	seriesCount := 10
+
+	// Append <seriesCount> series
+	app := db.Appender(context.Background())
+	for i := 0; i < seriesCount; i++ {
+		lset := labels.Labels{
+			{Name: model.MetricNameLabel, Value: fmt.Sprintf("series_%d", i)},
+		}
+		_, err := app.Append(0, lset, 0, 100)
+		require.NoError(t, err)
+	}
+	require.NoError(t, app.Commit())
+
+	// Truncate the WAL to force creation of a new segment.
+	require.NoError(t, db.truncate(0))
+	require.NoError(t, db.Close())
+
+	// Create a new storage and see what nextRef is initialized to.
+	db, err = Open(log.NewNopLogger(), nil, rs, dbDir, DefaultOptions())
+	require.NoError(t, err)
+	defer require.NoError(t, db.Close())
+
+	require.Equal(t, uint64(seriesCount), db.nextRef.Load(), "nextRef should be equal to the number of series written across the entire WAL")
+}
+
 func startTime() (int64, error) {
 	return time.Now().Unix() * 1000, nil
 }
@@ -420,8 +457,8 @@ func labelsForTest(lName string, seriesCount int) []labels.Labels {
 	for i := 0; i < seriesCount; i++ {
 		lset := labels.Labels{
 			{Name: "a", Value: lName},
-			{Name: "job", Value: "prometheus"},
 			{Name: "instance", Value: "localhost" + strconv.Itoa(i)},
+			{Name: "job", Value: "prometheus"},
 		}
 		series = append(series, lset)
 	}
@@ -445,3 +482,56 @@ func gatherFamily(t *testing.T, reg prometheus.Gatherer, familyName string) *dto
 
 	return nil
 }
+
+func TestStorage_DuplicateExemplarsIgnored(t *testing.T) {
+	s := createTestAgentDB(t, nil, DefaultOptions())
+	app := s.Appender(context.Background())
+	defer s.Close()
+
+	sRef, err := app.Append(0, labels.Labels{{Name: "a", Value: "1"}}, 0, 0)
+	require.NoError(t, err, "should not reject valid series")
+
+	// Write a few exemplars to our appender and call Commit().
+	// If the Labels, Value or Timestamp are different than the last exemplar,
+	// then a new one should be appended; Otherwise, it should be skipped.
+	e := exemplar.Exemplar{Labels: labels.Labels{{Name: "a", Value: "1"}}, Value: 20, Ts: 10, HasTs: true}
+	_, _ = app.AppendExemplar(sRef, nil, e)
+	_, _ = app.AppendExemplar(sRef, nil, e)
+
+	e.Labels = labels.Labels{{Name: "b", Value: "2"}}
+	_, _ = app.AppendExemplar(sRef, nil, e)
+	_, _ = app.AppendExemplar(sRef, nil, e)
+	_, _ = app.AppendExemplar(sRef, nil, e)
+
+	e.Value = 42
+	_, _ = app.AppendExemplar(sRef, nil, e)
+	_, _ = app.AppendExemplar(sRef, nil, e)
+
+	e.Ts = 25
+	_, _ = app.AppendExemplar(sRef, nil, e)
+	_, _ = app.AppendExemplar(sRef, nil, e)
+
+	require.NoError(t, app.Commit())
+
+	// Read back what was written to the WAL.
+	var walExemplarsCount int
+	sr, err := wal.NewSegmentsReader(s.wal.Dir())
+	require.NoError(t, err)
+	defer sr.Close()
+	r := wal.NewReader(sr)
+
+	var dec record.Decoder
+	for r.Next() {
+		rec := r.Record()
+		switch dec.Type(rec) {
+		case record.Exemplars:
+			var exemplars []record.RefExemplar
+			exemplars, err = dec.Exemplars(rec, exemplars)
+			require.NoError(t, err)
+			walExemplarsCount += len(exemplars)
+		}
+	}
+
+	// We had 9 calls to AppendExemplar but only 4 of those should have gotten through.
+	require.Equal(t, 4, walExemplarsCount)
+}
