@@ -566,6 +566,9 @@ func TestAgentAPIEndPoints(t *testing.T) {
 			resp, err := http.DefaultClient.Do(req)
 			require.NoError(t, err)
 			require.Equal(t, http.StatusUnprocessableEntity, resp.StatusCode)
+			t.Cleanup(func() {
+				require.NoError(t, resp.Body.Close())
+			})
 		}
 	}
 
@@ -584,6 +587,9 @@ func TestAgentAPIEndPoints(t *testing.T) {
 			resp, err := http.DefaultClient.Do(req)
 			require.NoError(t, err)
 			require.Equal(t, http.StatusOK, resp.StatusCode)
+			t.Cleanup(func() {
+				require.NoError(t, resp.Body.Close())
+			})
 		}
 	}
 }
